$ifndef _HITACHI_4_INC_
$define _HITACHI_4_INC_
'------------------------------------------------------------------------------------------------
' Replacement routine for the compiler's Print command for an Hitachi alphanumeric LCD
' Using a 4-bit interface, reversed bits for Arduino shields on an Amicus18 board
'
' The routine is for an 18F device only
'
    #disable Print                              ' Disable the compiler's print library routine
'
' Create system variables for the routine
'
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim BPF As Byte System
    Dim PP3 As Byte System
    Dim PP3H As Byte System
'
' Create some aliases to make the code's operation clearer
'
    Symbol Hitachi4_bByteOut = PP3
    Symbol Hitachi4_bWREGStore = PP3H
    Symbol Hitachi4_tComOrData = BPF.0
    Symbol Hitachi4_tInitialised = BPF.1

'------------------------------------------------------------------------------------------------
    Bra _Hitachi4_Main_                         ' Jump over the subroutine
'------------------------------------------------------------------------------------------------
Asm-
Print
Endasm-
    Hitachi4_bWREGStore = WREG                  ' Save the byte sent
    Low __LCD_ENPORT.__LCD_ENPIN                ' Pull the EN pin to output low
    Low __LCD_RSPORT.__LCD_RSPIN                ' Pull the RS pin to output Low
    #if(__LCD_DTPIN == 0)
        WREG = $F0
    #else
        WREG = $0F
    #endif
    Andwf (__LCD_DTPORT + 0x12)                 ' Set the correct half of the port to output
    WREG = Hitachi4_bWREGStore                  ' Get back the byte

    If Hitachi4_tInitialised = 0 Then           ' Has LCD been inititalised?
        DelayUS 15000                           ' No. So wait 15ms
        Hitachi4_bByteOut = $33                 ' Initialise the LCD
        Rcall Hitachi4_NibbleSend               ' Send init
        DelayUS 5000                            ' Wait 5ms
        Rcall Hitachi4_NibbleSend               ' Send init again
        DelayUS 100                             ' Wait 100us
        Rcall Hitachi4_NibbleSend               ' Send init one more time
        DelayUS 100                             ' Wait 100us
        Hitachi4_bByteOut = $22                 ' Set 4-bit mode
        Rcall Hitachi4_NibbleSend               ' Set the interface to 4-bit mode
        #ifdef __LCD_LINES                      ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)               ' Yes. So...
                WREG = $20                      ' Set for 4-bit mode, 1 line, 5x7 font
            #else
                WREG = $28                      ' Set for 4-bit mode, 2+ lines, 5x7 font
            #endif
        #else                                   ' Otherwise... Default to 2 lines
            WREG = $28                          ' Set for 4-bit mode, 2+ lines, 5x7 font
        #endif
        Rcall Hitachi4_SetCommand               ' Send command
        WREG = $0C                              ' Set for Display on, no cursor, no blink
        Rcall Hitachi4_SetCommand
        WREG = $06                              ' LCD entry mode set, increment, no shift
        Rcall Hitachi4_SetCommand
        Set Hitachi4_tInitialised               ' Indicate that the LCD is initialised
        WREG = Hitachi4_bWREGStore              ' Get the saved byte back
        Bra Hitachi4_Skip
Hitachi4_SetCommand:
        Set Hitachi4_tComOrData                 ' Set for command
    EndIf
Hitachi4_Skip:
    Hitachi4_bByteOut = WREG                    ' Get the byte to send from WREG
    If Hitachi4_tComOrData = 1 Then             ' Is it a command this time?
        Clear __LCD_RSPORT.__LCD_RSPIN          ' Yes. So set command register select
        Sublw 3
        Bnc Hitachi4_SetData                    ' Short delay
        Rcall Hitachi4_SetData                  ' Long delay
        DelayUS __LCD_COMMANDUS                 ' Wait for the command to complete
        Return
    EndIf
    Set Hitachi4_tComOrData                     ' Indicate first nibble, or command next time
    Sublw $FE                                   ' Command next time? (the byte is still in WREG)
    Bz Hitachi4_Exit
    Set __LCD_RSPORT.__LCD_RSPIN                ' Set the RS pin

Hitachi4_SetData:
    #if(__LCD_DTPIN == 0)
        Swapf Hitachi4_bByteOut,f               ' Swap top and bottom nibbles
    #endif
    Btfss Hitachi4_tComOrData                   ' First time through only
Hitachi4_NibbleSend:
    Clear Hitachi4_tComOrData                   ' Indicate second nibble, and data next time
    Set __LCD_ENPORT.__LCD_ENPIN                ' Enabled
    #if(__LCD_DTPIN == 0)
        WREG = $F0                              ' Clear the bits of the LCD data port
    #else
        WREG = $0F                              ' Clear the bits of the LCD data port
    #endif
    Andwf __LCD_DTPORT,f
    '
    ' Reverse the bits of the port
    '
    PP0 = Hitachi4_bByteOut                     ' Load the byte to be rotated
    WREG = 4                                    ' Create a loop for 4-bits
Hitachi4_RevLoop:
    Ror PP0                                     ' Rotate LSB of PP0 into the Carry
    Rol PP0H                                    ' Rotate the Carry into LSB of PP0H
    Decfsz WREG,f                               ' \
    Bra Hitachi4_RevLoop                        ' / Do it for all 4-bits
    WREG = PP0H                                 ' Isolate correct nibble of the byte
    #if(__LCD_DTPIN == 0)
        Andlw $0F
    #else
        Andlw $F0
    #endif
    Iorwf __LCD_DTPORT,f                        ' Write the byte to the port

    DelayCS 6
    Clear __LCD_ENPORT.__LCD_ENPIN              ' Disabled
    Swapf Hitachi4_bByteOut,f                   ' Move to the other nibble
    DelayCS 6
    Btfsc Hitachi4_tComOrData
    Bra Hitachi4_NibbleSend                     ' Send the lower 4-bits to the LCD
    DelayUS __LCD_DATAUS                        ' Wait for the data operation to complete
    DelayUS __LCD_DATAUS                        ' Wait for the command to complete
Hitachi4_Exit:
    WREG = Hitachi4_bWREGStore                  ' Return the original byte in WREG
    Return

'------------------------------------------------------------------------------------------------
_Hitachi4_Main_:
$endif  ' _HITACHI_4_INC_
