$ifndef _HITACHI_8_INC_
$define _HITACHI_8_INC_
'------------------------------------------------------------------------------------------------
' Replacement routine for the compiler's Print command for an Hitachi alphanumeric LCD
' Using an 8-bit interface, reversed bits for Arduino shields on an Amicus18 board
'
' The routine is for an 18F device only
'
    #disable Print                              ' Disable the compiler's print library routine
'
' Create system variables for the routine
'
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim BPF As Byte System
    Dim PP3 As Byte System
    Dim PP3H As Byte System
'
' Create some aliases to make the code's operation clearer
'
    Symbol Hitachi8_bByteOut = PP3
    Symbol Hitachi8_bWREGStore = PP3H
    Symbol Hitachi8_tComOrData = BPF.0
    Symbol Hitachi8_tInitialised = BPF.1

'------------------------------------------------------------------------------------------------
    Bra _Hitachi8_Main_                         ' Jump over the subroutine
'------------------------------------------------------------------------------------------------
Asm-
Print
Endasm-
    Hitachi8_bWREGStore = WREG                  ' Save the byte
    Low __LCD_ENPORT.__LCD_ENPIN                ' Pull the EN pin to output low
    Low __LCD_RSPORT.__LCD_RSPIN                ' Pull the RS pin to output Low
    Output __LCD_DTPORT                         ' Set the port to output

    If Hitachi8_tInitialised = 0 Then           ' Has the LCD been inititalised?
        DelayUS 15000                           ' No. So wait at least 15ms
        Hitachi8_bByteOut = $33                 ' Initialise the LCD
        Rcall Hitachi8_SetData                  ' Send init
        DelayUS 5000                            ' Wait at least 5ms
        Rcall Hitachi8_SetData                  ' Send init again
        DelayUS 100                             ' Wait at least 100us
        Rcall Hitachi8_SetData                  ' Send init one more time
        DelayUS 100                             ' Wait at least 100us
        #ifdef __LCD_LINES                      ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)               ' Yes. So...
                WREG = $30                      ' Set 8-bit mode, 1 line, 5x7 font
            #else
                WREG = $38                      ' Set 8-bit mode, 2+ lines, 5x7 font
            #endif
        #else                                   ' Otherwise... Default to 2 lines
            WREG = $38                          ' Set 8-bit mode, 2+ lines, 5x7 font
        #endif
        Rcall Hitachi8_SetCommand               ' Send the command
        WREG = $0C                              ' Display on, no cursor,no blink
        Rcall Hitachi8_SetCommand
        WREG = $06                              ' LCD entry mode set,increment,no shift
        Rcall Hitachi8_SetCommand
        Set Hitachi8_tInitialised               ' Indicate that the LCD has been initialised
        WREG = Hitachi8_bWREGStore              ' Get te saved byte back
        Bra Hitachi8_SkipInit
Hitachi8_SetCommand:
        Set Hitachi8_tComOrData                 ' Set for command
    EndIf

Hitachi8_SkipInit:
    Hitachi8_bByteOut = WREG                    ' Get the byte to send from WREG
    If Hitachi8_tComOrData = 1 Then             ' Command this time?
        Clear __LCD_RSPORT.__LCD_RSPIN          ' Yes. So clear the RS pin
        Sublw 3
        Bnc Hitachi8_SetData                    ' Short delay
        Rcall Hitachi8_SetData                  ' Long delay
        DelayUS __LCD_COMMANDUS                 ' Wait for the command to complete
        Return
    EndIf
    Set Hitachi8_tComOrData                     ' Indicate a command next time
    Sublw 254                                   ' Command next time? (byte is still in WREG)
    Bz Hitachi8_Exit
    Set __LCD_RSPORT.__LCD_RSPIN                ' Set the RS pin

Hitachi8_SetData:
    Set __LCD_ENPORT.__LCD_ENPIN                ' Enabled
    '
    ' Reverse the bits of the port
    '
    PP0 = Hitachi8_bByteOut                     ' Load the byte to be rotated
    WREG = 8                                    ' Create a loop for 8-bits
Hitachi8_RevLoop:
    Ror PP0                                     ' Rotate LSB of PP0 into Carry
    Rol PP0H                                    ' Rotate Carry into LSB of PP0H
    Decfsz WREG,f                               ' \
    Bra Hitachi8_RevLoop                        ' / Do it for all 8-bits
    __LCD_DTPORT = PP0H                         ' Write the byte to the port
    DelayCS 6
    Clear Hitachi8_tComOrData                   ' Indicate data next time
    Clear __LCD_ENPORT.__LCD_ENPIN              ' Disabled
    DelayUS __LCD_DATAUS                        ' Wait for the data operation to complete
Hitachi8_Exit:
    WREG = Hitachi8_bWREGStore                  ' Return the original byte in WREG
    Return

'------------------------------------------------------------------------------------------------
_Hitachi8_Main_:
$endif  ' _HITACHI_8_INC_

